/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */

#ifndef INCLUDED_VCL_INC_IOS_IOSINST_HXX
#define INCLUDED_VCL_INC_IOS_IOSINST_HXX

#include <premac.h>
#include <CoreGraphics/CoreGraphics.h>
#include <postmac.h>

#include <tools/link.hxx>

#include "headless/svpinst.hxx"
#include "headless/svpframe.hxx"

class IosSalFrame;
class SystemFontList;

class IosSalInstance : public SvpSalInstance
{
public:
    IosSalInstance(std::unique_ptr<SalYieldMutex> pMutex);
    virtual ~IosSalInstance();
    static IosSalInstance* getInstance();

    SalSystem* CreateSalSystem() override;

    css::uno::Reference<css::uno::XInterface>
    CreateClipboard(const css::uno::Sequence<css::uno::Any>& i_rArguments) override;

    void GetWorkArea(AbsoluteScreenPixelRectangle& rRect);
    SalFrame* CreateFrame(SalFrame* pParent, SalFrameStyleFlags nStyle) override;
    SalFrame* CreateChildFrame(SystemParentData* pParent, SalFrameStyleFlags nStyle) override;
};

class SalData
{
public:
    std::unique_ptr<SystemFontList> mpFontList;
    CGColorSpaceRef mxRGBSpace;
    CGColorSpaceRef mxGraySpace;

    static void ensureThreadAutoreleasePool(){};

    explicit SalData();
    virtual ~SalData();
};

#endif // INCLUDED_VCL_INC_IOS_IOSINST_HXX

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
